@@ -11,6 +11,41 @@ from .constants import *
 __all__ = [ 'Distribution',
             'Beam' ]
 
+def read_distribution_file(fname):
+    ''' Read distribution from file .csv
+
+    '''
+    cols = ['x', 'y', 'z', 'px', 'py', 'pz']
+    #        m    m    m    MeV/c  MeV/c  MeV/c
+
+    df = pd.read_csv(fname, header=None, delim_whitespace=True, names=cols, dtype='float32')
+    return df['x'], df['y'], df['z'], df['px'], df['py'], df['pz']
+
+
+def read_distribution_astra(fname):
+    ''' Read distribution from Astra file
+
+    '''
+    cols = ['x', 'y', 'z', 'px', 'py', 'pz', 'clock', 'charge', 'id', 'flag']
+    #        m    m    m    eV/c  eV/c  eV/c  ns       nC
+    df = pd.read_csv(fname, header=None, delim_whitespace=True, names=cols, dtype='float32')
+
+    df = df[df.flag != -15] # ignore the lost particles
+    df['px'] = df['px']/1e6 # MeV/c
+    df['py'] = df['py']/1e6 # MeV/c
+
+    # remove the reference particle
+    df0 = df.head(1)
+    df  = df.drop(df0.index)
+
+    z0  = df0.z.values[0]
+    pz0 = df0.pz.values[0]
+
+    # Recalculate z and pz:
+    df['z'] = z0 + df['clock']*1e-9*c # m
+    df['pz'] = (pz0 + df['pz'])/1e6 # MeV/c
+    return df['x'], df['y'], df['z'], df['px'], df['py'], df['pz']
+
 Distribution = namedtuple('Distribution', [ 'name', 'x', 'y', 'z', 'px', 'py', 'pz' ])
 
 class Beam:
@@ -20,7 +55,7 @@ class Beam:
 
     def __init__(self, type: Element, *, charge: float=0.0) -> None:
         self.type = type        # particles type
-        self.charge = charge
+        self.charge = charge    # beam charge
         self.n = 0.0            # quantity
         self.da = np.array      # data array
         self.df = pd.DataFrame  # data frame
@@ -31,10 +66,11 @@ class Beam:
 
         This function generates a beam with a given distribution and initial beam displacement.
         '''
-        assert n > 0, 'The number of particles (n) must be a positive number!'
-        assert type(distribution.name) == str, 'Distribution name must be a string!'
         condition = ((distribution.name == 'KV' or distribution.name == 'Uniform') or
                     (distribution.name == 'Gauss' or distribution.name == 'GA'))
+
+        assert n > 0, 'The number of particles (n) must be a positive number!'
+        assert type(distribution.name) == str, 'Distribution name must be a string!'
         assert condition, 'Сheck distribution name!'
 
         self.n = n
@@ -58,21 +94,24 @@ class Beam:
         self.df = pd.DataFrame(np.transpose(self.da), columns=['x','y','z','px','py','pz'])
         self.df.to_csv(self.type.symbol + 'Beam.csv')
 
-    def upload_particles(self, x: np.array, y: np.array, z: np.array,
-                         px: np.array, py: np.array, pz: np.array) -> None:
+    def upload(self, file_name: str):
         ''' Particle loading
 
-        Y = np.array[[ ... ] # x[m]
-                     [ ... ] # y[m]
-                     [ ... ] # z[m]
-                     [ ... ] # px[MeV/c]
-                     [ ... ] # py[MeV/c]
-                     [ ... ] # pz[MeV/c]
-                     ]
         '''
-        self.da = np.row_stack((x, y, z, px, py, pz))
-        self.df = pd.DataFrame(np.transpose(self.da), columns=['x','y','z','px','py','pz'])
-        self.df.to_csv(self.type.symbol + 'Beam.csv')
+        file_extension = file_name.split('.')[-1]
+
+        if file_extension == 'ini':
+            x, y, z, px, py, pz = read_distribution_astra(file_name)
+            self.n = int(len(x))
+            self.da = np.row_stack((x, y, z, px, py, pz))
+            self.df = pd.DataFrame(np.transpose(self.da), columns=['x','y','z','px','py','pz'])
+            self.df.to_csv(self.type.symbol + 'Beam.csv')
+        if file_extension == 'csv':
+            x, y, z, px, py, pz = read_distribution_file(file_name)
+            self.n = int(len(x))
+            self.da = np.row_stack((x, y, z, px, py, pz))
+            self.df = pd.DataFrame(np.transpose(self.da), columns=['x','y','z','px','py','pz'])
+            self.df.to_csv(self.type.symbol + 'Beam.csv')
 
     def __str__(self):
         return str(self.df)
